import { isArray, sortBy, uniq } from 'lodash-es';
import * as React from 'react';
import { Subscription } from 'rxjs';
import stringScore from 'string-score';
import { Key } from 'ts-key-enum';
import { HighlightedMatches } from '../ui/generic/HighlightedMatches';
import { PopoverButton } from '../ui/generic/PopoverButton';
import { ActionItem } from './actions/ActionItem';
import { getContributedActionItems } from './actions/contributions';
/** Displays a list of commands contributed by extensions for a specific menu. */
export class CommandList extends React.PureComponent {
    constructor() {
        super(...arguments);
        this.state = { input: '', selectedIndex: 0, recentActions: CommandList.readRecentActions() };
        this.subscriptions = new Subscription();
        this.selectedItem = null;
        this.setSelectedItem = (e) => (this.selectedItem = e);
        this.onInputChange = e => {
            this.setState({ input: e.currentTarget.value, selectedIndex: 0 });
        };
        this.onInputKeyDown = e => {
            switch (e.key) {
                case Key.ArrowDown: {
                    e.preventDefault();
                    this.setSelectedIndex(1);
                    break;
                }
                case Key.ArrowUp: {
                    e.preventDefault();
                    this.setSelectedIndex(-1);
                    break;
                }
                case Key.Enter: {
                    if (this.selectedItem) {
                        this.selectedItem.runAction(e);
                    }
                    break;
                }
            }
        };
        this.onSubmit = e => e.preventDefault();
        this.onActionRun = (actionID) => {
            const KEEP_RECENT_ACTIONS = 10;
            this.setState(prevState => {
                const { recentActions } = prevState;
                if (!recentActions) {
                    return { recentActions: [actionID] };
                }
                return { recentActions: uniq([actionID, ...recentActions]).slice(0, KEEP_RECENT_ACTIONS) };
            });
            if (this.props.onSelect) {
                this.props.onSelect();
            }
        };
    }
    static readRecentActions() {
        const value = localStorage.getItem(CommandList.RECENT_ACTIONS_STORAGE_KEY);
        if (value === null) {
            return null;
        }
        try {
            const recentActions = JSON.parse(value);
            if (isArray(recentActions) && recentActions.every(a => typeof a === 'string')) {
                return recentActions;
            }
            return null;
        }
        catch (err) {
            console.error('Error reading recent actions:', err);
        }
        CommandList.writeRecentActions(null);
        return null;
    }
    static writeRecentActions(recentActions) {
        try {
            if (recentActions === null) {
                localStorage.removeItem(CommandList.RECENT_ACTIONS_STORAGE_KEY);
            }
            else {
                const value = JSON.stringify(recentActions);
                localStorage.setItem(CommandList.RECENT_ACTIONS_STORAGE_KEY, value);
            }
        }
        catch (err) {
            console.error('Error writing recent actions:', err);
        }
    }
    componentDidMount() {
        this.subscriptions.add(this.props.extensionsController.registries.contribution
            .getContributions()
            .subscribe(contributions => this.setState({ contributions })));
    }
    componentDidUpdate(_prevProps, prevState) {
        if (this.state.recentActions !== prevState.recentActions) {
            CommandList.writeRecentActions(this.state.recentActions);
        }
    }
    componentWillUnmount() {
        this.subscriptions.unsubscribe();
    }
    render() {
        if (!this.state.contributions) {
            return null;
        }
        const allItems = getContributedActionItems(this.state.contributions, this.props.menu);
        // Filter and sort by score.
        const query = this.state.input.trim();
        const items = filterAndRankItems(allItems, this.state.input, this.state.recentActions);
        // Support wrapping around.
        const selectedIndex = ((this.state.selectedIndex % items.length) + items.length) % items.length;
        return (React.createElement("div", { className: "command-list list-group list-group-flush rounded" },
            React.createElement("div", { className: "list-group-item" },
                React.createElement("form", { className: "form", onSubmit: this.onSubmit },
                    React.createElement("label", { className: "sr-only", htmlFor: "command-list__input" }, "Command"),
                    React.createElement("input", { id: "command-list__input", type: "text", className: "form-control px-2 py-1 rounded-0", value: this.state.input, placeholder: "Command...", spellCheck: false, autoCorrect: "off", autoComplete: "off", autoFocus: true, onChange: this.onInputChange, onKeyDown: this.onInputKeyDown }))),
            items.length > 0 ? (items.map((item, i) => (React.createElement(ActionItem, Object.assign({ className: `list-group-item list-group-item-action px-3 ${i === selectedIndex ? 'active border-primary' : ''}`, key: i }, item, { ref: i === selectedIndex ? this.setSelectedItem : undefined, title: React.createElement(HighlightedMatches, { text: `${item.action.category ? `${item.action.category}: ` : ''}${item.action
                        .title || item.action.command}`, pattern: query }), onRun: this.onActionRun, extensionsController: this.props.extensionsController, extensions: this.props.extensions, location: this.props.location }))))) : (React.createElement("div", { className: "list-group-item text-muted bg-striped-secondary" }, "No matching commands"))));
    }
    setSelectedIndex(delta) {
        this.setState(prevState => ({ selectedIndex: prevState.selectedIndex + delta }));
    }
}
// Persist recent actions in localStorage. Be robust to serialization errors.
CommandList.RECENT_ACTIONS_STORAGE_KEY = 'commandList.recentActions';
export function filterAndRankItems(items, query, recentActions) {
    if (!query) {
        if (recentActions === null) {
            return items;
        }
        // Show recent actions first.
        return sortBy(items, (item) => {
            const index = recentActions.indexOf(item.action.id);
            return index === -1 ? null : index;
        }, ({ action }) => action.id);
    }
    // Memoize labels and scores.
    const labels = new Array(items.length);
    const scores = new Array(items.length);
    const scoredItems = items
        .filter((item, i) => {
        let label = labels[i];
        if (label === undefined) {
            label = `${item.action.category ? `${item.action.category}: ` : ''}${item.action.title ||
                item.action.command}`;
            labels[i] = label;
        }
        if (scores[i] === undefined) {
            scores[i] = stringScore(label, query, 0);
        }
        return scores[i] > 0;
    })
        .map((item, i) => {
        const index = recentActions && recentActions.indexOf(item.action.id);
        return { item, score: scores[i], recentIndex: index === -1 ? null : index };
    });
    return sortBy(scoredItems, 'recentIndex', 'score', ({ item }) => item.action.id).map(({ item }) => item);
}
export class CommandListPopoverButton extends React.PureComponent {
    constructor() {
        super(...arguments);
        this.state = {};
        this.dismissPopover = () => this.setState({ hideOnChange: {} });
    }
    render() {
        return (React.createElement(PopoverButton, { caretIcon: this.props.extensions.context.icons.CaretDown, popoverClassName: "rounded", placement: "auto-end", toggleVisibilityKeybinding: this.props.toggleVisibilityKeybinding, hideOnChange: this.state.hideOnChange, popoverElement: React.createElement(CommandList, Object.assign({}, this.props, { onSelect: this.dismissPopover })) },
            React.createElement(this.props.extensions.context.icons.Menu, { className: "icon-inline" })));
    }
}
//# sourceMappingURL=CommandList.js.map