import * as React from 'react';
import { from, Subject, Subscription } from 'rxjs';
import { catchError, map, mapTo, mergeMap, startWith, tap } from 'rxjs/operators';
import { urlForOpenPanel } from '../../client/clientCommands';
import { asError } from '../../errors';
import { LinkOrButton } from '../../ui/generic/LinkOrButton';
const LOADING = 'loading';
export class ActionItem extends React.PureComponent {
    constructor() {
        super(...arguments);
        this.state = { actionOrError: null };
        this.commandExecutions = new Subject();
        this.subscriptions = new Subscription();
        this.runAction = (e) => {
            const action = (isAltEvent(e) && this.props.altAction) || this.props.action;
            if (urlForClientCommandOpen(action, this.props.location)) {
                if (e.currentTarget.tagName === 'A' && e.currentTarget.hasAttribute('href')) {
                    // Do not execute the command. The <LinkOrButton>'s default event handler will do what we want (which
                    // is to open a URL). The only case where this breaks is if both the action and alt action are "open"
                    // commands; in that case, this only ever opens the (non-alt) action.
                    if (this.props.onRun) {
                        // Defer calling onRun until after the URL has been opened. If we call it immediately, then in
                        // CommandList it immediately updates the (most-recent-first) ordering of the ActionItems, and
                        // the URL actually changes underneath us before the URL is opened. There is no harm to
                        // deferring this call; onRun's documentation allows this.
                        const onRun = this.props.onRun;
                        setTimeout(() => onRun(action.id));
                    }
                    return;
                }
            }
            // If the action we're running is *not* opening a URL by using the event target's default handler, then
            // ensure the default event handler for the <LinkOrButton> doesn't run (which might open the URL).
            e.preventDefault();
            this.commandExecutions.next({
                command: this.props.action.command,
                arguments: this.props.action.commandArguments,
            });
        };
    }
    componentDidMount() {
        this.subscriptions.add(this.commandExecutions
            .pipe(mergeMap(params => from(this.props.extensionsController.executeCommand(params)).pipe(mapTo(null), catchError(error => [asError(error)]), map(c => ({ actionOrError: c })), tap(() => {
            if (this.props.onRun) {
                this.props.onRun(this.props.action.id);
            }
        }), startWith({ actionOrError: LOADING }))))
            .subscribe(stateUpdate => this.setState(stateUpdate), error => console.error(error)));
    }
    componentDidUpdate(prevProps, prevState) {
        // If the tooltip changes while it's visible, we need to force-update it to show the new value.
        const prevTooltip = prevProps.action.actionItem && prevProps.action.actionItem.description;
        const tooltip = this.props.action.actionItem && this.props.action.actionItem.description;
        if (prevTooltip !== tooltip) {
            this.props.extensions.context.forceUpdateTooltip();
        }
    }
    componentWillUnmount() {
        this.subscriptions.unsubscribe();
    }
    render() {
        let content;
        let tooltip;
        if (this.props.title) {
            content = this.props.title;
            tooltip = this.props.action.description;
        }
        else if (this.props.variant === 'actionItem' && this.props.action.actionItem) {
            content = (React.createElement(React.Fragment, null,
                this.props.action.actionItem.iconURL && (React.createElement("img", { src: this.props.action.actionItem.iconURL, alt: this.props.action.actionItem.iconDescription, className: "icon-inline" })),
                ' ',
                this.props.action.actionItem.label));
            tooltip = this.props.action.actionItem.description;
        }
        else {
            content = (React.createElement(React.Fragment, null,
                this.props.action.iconURL && React.createElement("img", { src: this.props.action.iconURL, className: "icon-inline" }),
                ' ',
                this.props.action.category ? `${this.props.action.category}: ` : '',
                this.props.action.title));
            tooltip = this.props.action.description;
        }
        return (React.createElement(LinkOrButton, { "data-tooltip": tooltip, disabled: this.props.disabledDuringExecution && this.state.actionOrError === LOADING, className: this.props.className, 
            // If the command is 'open' or 'openXyz' (builtin commands), render it as a link. Otherwise render
            // it as a button that executes the command.
            to: urlForClientCommandOpen(this.props.action, this.props.location) ||
                (this.props.altAction && urlForClientCommandOpen(this.props.altAction, this.props.location)), onSelect: this.runAction }, content));
    }
}
function urlForClientCommandOpen(action, location) {
    if (action.command === 'open' && action.commandArguments && typeof action.commandArguments[0] === 'string') {
        return action.commandArguments[0];
    }
    if (action.command === 'openPanel' && action.commandArguments && typeof action.commandArguments[0] === 'string') {
        return urlForOpenPanel(action.commandArguments[0], location.hash);
    }
    return undefined;
}
function isAltEvent(e) {
    return e.altKey || e.metaKey || e.ctrlKey || ('button' in e && e.button === 1);
}
//# sourceMappingURL=ActionItem.js.map