import { cloneDeep, isFunction, isPlainObject } from 'lodash-es';
import { createAggregateError, isErrorLike } from './errors';
import { parseJSONCOrError } from './util';
/** Converts a GraphQL ConfigurationCascade value to a value of this library's ConfigurationCascade type. */
export function gqlToCascade({ subjects, }) {
    const cascade = {
        subjects: [],
        merged: null,
    };
    const allSettings = [];
    const allSettingsErrors = [];
    for (const subject of subjects) {
        const settings = subject.latestSettings && parseJSONCOrError(subject.latestSettings.configuration.contents);
        cascade.subjects.push({ subject, settings });
        if (isErrorLike(settings)) {
            allSettingsErrors.push(settings);
        }
        else if (settings !== null) {
            allSettings.push(settings);
        }
    }
    if (allSettingsErrors.length > 0) {
        cascade.merged = createAggregateError(allSettingsErrors);
    }
    else {
        cascade.merged = mergeSettings(allSettings);
    }
    return cascade;
}
/**
 * Deeply merges the settings without modifying any of the input values. The array is ordered from lowest to
 * highest precedence in the merge.
 *
 * TODO(sqs): In the future, this will pass a CustomMergeFunctions value to merge.
 */
export function mergeSettings(values) {
    if (values.length === 0) {
        return null;
    }
    const target = cloneDeep(values[0]);
    for (const value of values.slice(1)) {
        merge(target, value);
    }
    return target;
}
/**
 * Deeply merges add into base (modifying base). The merged value for a key path can be customized by providing a
 * function at the same key path in custom.
 *
 * Most callers should use mergeSettings, which uses the set of CustomMergeFunctions that are required to properly
 * merge settings.
 */
export function merge(base, add, custom) {
    for (const key of Object.keys(add)) {
        if (key in base) {
            const customEntry = custom && custom[key];
            if (customEntry && isFunction(customEntry)) {
                base[key] = customEntry(base[key], add[key]);
            }
            else if (isPlainObject(base[key]) && isPlainObject(add[key])) {
                merge(base[key], add[key], customEntry);
            }
            else {
                base[key] = add[key];
            }
        }
        else {
            base[key] = add[key];
        }
    }
}
/**
 * The conventional ordering of extension configuration subject types in a list.
 */
export const SUBJECT_TYPE_ORDER = ['Client', 'User', 'Org', 'Site'];
export function subjectTypeHeader(nodeType) {
    switch (nodeType) {
        case 'Client':
            return null;
        case 'Site':
            return null;
        case 'Org':
            return 'Organization:';
        case 'User':
            return null;
    }
}
export function subjectLabel(subject) {
    switch (subject.__typename) {
        case 'Client':
            return 'Client';
        case 'Site':
            return 'Everyone';
        case 'Org':
            return subject.name;
        case 'User':
            return subject.username;
    }
}
//# sourceMappingURL=settings.js.map