import * as React from 'react';
import { Key } from 'ts-key-enum';
import { RouterLinkOrAnchor } from './RouterLinkOrAnchor';
/**
 * A component that is displayed in the same way, regardless of whether it's a link (with a
 * destination URL) or a button (with a click handler).
 *
 * It is keyboard accessible: unlike <Link> or <a>, pressing the enter key triggers it. Unlike
 * <button>, it shows a focus ring.
 */
export class LinkOrButton extends React.PureComponent {
    constructor() {
        super(...arguments);
        this.onAnchorClick = e => {
            if (this.props.onSelect) {
                this.props.onSelect(e);
            }
        };
        this.onAnchorKeyPress = e => {
            if (isSelectKeyPress(e)) {
                if (this.props.onSelect) {
                    this.props.onSelect(e);
                }
            }
        };
    }
    render() {
        const className = `${this.props.className === undefined ? 'nav-link' : this.props.className} ${this.props.disabled ? 'disabled' : ''}`;
        const commonProps = {
            className,
            'data-tooltip': this.props['data-tooltip'],
            'aria-label': this.props['data-tooltip'],
            tabIndex: 0,
            onClick: this.onAnchorClick,
            onKeyPress: this.onAnchorKeyPress,
        };
        if (!this.props.to) {
            // Render using an <a> with no href, so that we get a focus ring (when using Bootstrap).
            // We need to set up a keypress listener because <a onclick> doesn't get triggered by
            // enter.
            return React.createElement("a", Object.assign({}, commonProps), this.props.children);
        }
        return (React.createElement(RouterLinkOrAnchor, Object.assign({}, commonProps, { to: this.props.to, target: this.props.target }), this.props.children));
    }
}
function isSelectKeyPress(e) {
    return e.key === Key.Enter && !e.ctrlKey && !e.shiftKey && !e.metaKey && !e.altKey;
}
//# sourceMappingURL=LinkOrButton.js.map